/*********************************************
 *
 * Affine Mode, Demo 2
 *
 *  Based on parasytes rotate demo
 *
 *  By david Tucker
 *  Rev: 0.9, 8/25/05 - initial release
 *
 * An attempt at a pole position style effect
 * needs lost of work, mostly in the projection algorithem
 * also the grass is not working as advertised.
 *
 **********************************************/
//gccvb includes
#include "types.h"
#include "video.h"
#include "joypad.h"
#include "world.h"
#include "mem.h"

//dt includes
#include "dt_math.h"
#include "dt_affine.h"

//images
#include "focus_map.h"
#include "road_map.h"

//controls the depth of the turn
#define maxalpha 256

#define SCREEN_WIDTH	384
#define SCREEN_HEIGHT	224

#define SKY_WIDTH		512
#define SKY_HEIGHT		64
#define SKY_OFFSET		256

#define TRACK_1_WIDTH	384
#define TRACK_1_HEIGHT	128

#define TRACK_2_WIDTH	512
#define TRACK_2_HEIGHT	TRACK_1_HEIGHT //must be the same height

//memory base of the param table, don't overwrite with char data!
u16* const param_b = (u16*)0x0003C000+((SCREEN_HEIGHT-SKY_HEIGHT)*2);
u16* const param_t = (u16*)0x0003C000;

//Display the focus screen and wait for keyboard input
void focus_screen() {
	int i;

	//initiate left focus screen
	WA[31].head = WRLD_LON;
	WA[31].gx = 0;
	WA[31].gp = 0;
	WA[31].gy = 0;
	WA[31].mx = 0;
	WA[31].mp = 0;
	WA[31].my = 0;
	WA[31].w  = SCREEN_WIDTH;
	WA[31].h  = SCREEN_HEIGHT;

	//initiate right focus screen
	WA[30].head = WRLD_RON;
	WA[30].gx = 0;
	WA[30].gp = 0;
	WA[30].gy = 0;
	WA[30].mx = 0;
	WA[30].mp = 0;
	WA[30].my = SCREEN_HEIGHT;
	WA[30].w  = SCREEN_WIDTH;
	WA[30].h  = SCREEN_HEIGHT;

	//last world
	WA[29].head = WRLD_END;

	//move image to char table and bgmap
	copymem((BYTE*)CharSeg0, focus_char, focus_char_len);
	copymem((BYTE*)BGMMBase, focus_map, focus_map_len);

	//turn on display
	vbDisplayOn();
	vbDisplayHide();

	vbFXFadeIn(1);

	//do we need to put a delay in?
	while(! vbPadKeyDown()) ;
	while(! vbPadKeyUp()  ) ;

	vbFXFadeOut(1);
}

//configure for affine mode
void setup_affine() {
	//clear screen and set things up
	vbDisplayHide();

	//initiate world as affine mode
	//track background //***FixMe, not working.
	WA[31].head = WRLD_AFFINE ; //WRLD_ON
	WA[31].gx = 0;
	WA[31].gp = 0;
	WA[31].gy = SKY_HEIGHT;
	WA[31].mx = 0;
	WA[31].mp = 0;
	WA[31].my = 0;
	WA[31].w = SCREEN_WIDTH;
	WA[31].h = SCREEN_HEIGHT-SKY_HEIGHT;
	WA[31].param = (u32)param_b;
	WA[31].ovr = 0x00; //blank overplane character, kills BG tiling
	
	//track lines
	WA[30].head = WRLD_ON | WRLD_AFFINE | WRLD_OVR;
	WA[30].gx = 0;
	WA[30].gp = 0;
	WA[30].gy = SKY_HEIGHT;
	WA[30].mx = 0;
	WA[30].mp = 0;
	WA[30].my = 0;
	WA[30].w = SCREEN_WIDTH;
	WA[30].h = SCREEN_HEIGHT-SKY_HEIGHT;
	WA[30].param = (u32)param_t;
	WA[30].ovr = 0x00; //blank overplane character, kills BG tiling

	//sky
	WA[29].head = WRLD_ON | WRLD_BGMAP;
	WA[29].gx = 0;
	WA[29].gp = 0;
	WA[29].gy = 0;
	WA[29].mx = 0;
	WA[29].mp = 0;
	WA[29].my = SKY_OFFSET;
	WA[29].w = SCREEN_WIDTH;
	WA[29].h = SKY_HEIGHT;

	WA[28].head = WRLD_END;

	//move image to char table and bgmap
	copymem((BYTE*)CharSeg0, road_char, road_char_len);
	copymem((BYTE*)BGMMBase, road_map, road_map_len);

	//restore display, move me??
	vbWaitFrame(1);
	vbDisplayShow();
}

//cheap fn to compute a cured track, replace with a real
// projection function someday
#define MAX_C 128
#define MAX_S 400
int curve(int offset) {
	offset*=0.1f;
	if(offset<0) return 0;
	if(offset> MAX_C) {
		return ((1-COSF(MAX_C-1))+((1-COSF(MAX_C))-(1-COSF(MAX_C-1)))*MAX_S)*(offset-MAX_C);
	}
	return (int)((1-COSF(offset))*MAX_S);
}

int main () {
	PDx_ST pdx;
	s16 alpha = 0;
	s16 offset = 0;
	int i, max,cx, cz;
	float cy,t;
	AFFINE_ST *affine_b, *affine_t;
	int inc = 1;

	//display focus screen
	focus_screen();

	//prepare everything for affine mode
	setup_affine();


	//ToDo, Get some real math in hear to compute the curve
	// and road projections.
	//main loop
	while (1) {
		//controll our turn
		if((alpha += inc) > maxalpha) inc =-1;
		else if(alpha<=0) inc = 1;

		//controll our speed
		offset += 8;
		//offset &= (TRACK_1_HEIGHT-1); //the road segment wraps

		//place ourselves over the track
		cx = TRACK_1_WIDTH/2;	//horizontal center on bitmap
		cz = TRACK_1_HEIGHT;	//sit over start of bitmap
		cy = 75;				//altitude above track

		//compute road background and lane stripes
		affine_b = (AFFINE_ST*)((WA[31].param<<1)+0x00020000);
		affine_t = (AFFINE_ST*)((WA[30].param<<1)+0x00020000);
		max = WA[31].h;
		for (i = 0; i < max; i++) {
			pdx.pa = (cy/i);//*COSF(alpha);
			pdx.pd = (cy/i);//*SINF(alpha);

			pdx.dx = cx-(pdx.pa*(SCREEN_WIDTH/2+curve((alpha-(i*2))*3)));
			pdx.dy = cz-(pdx.pd*(SCREEN_HEIGHT/2));

			affine_b[i].pb_y    = FTOFIX13_3(pdx.dx);
			affine_b[i].paralax = 0;
			affine_b[i].pd_y    = FTOFIX13_3(i*pdx.pd+pdx.dy-offset)
				&((TRACK_2_HEIGHT<<3)-1)+ITOFIX13_3(TRACK_1_HEIGHT);

			affine_b[i].pa      = FTOFIX7_9(pdx.pa);
			affine_b[i].pc      = 0;

			affine_t[i].pb_y    = FTOFIX13_3(pdx.dx);
			affine_t[i].paralax = 0;
			affine_t[i].pd_y    = FTOFIX13_3(i*pdx.pd+pdx.dy-offset)
				&((TRACK_1_HEIGHT<<3)-1) ; //((64*8)-1), convert to Fix point

			affine_t[i].pa      = FTOFIX7_9(pdx.pa);
			affine_t[i].pc      = 0;
		}


		//delay 1/50th of a second
		vbWaitFrame(1);
	}
}
